/*------------------------------------------------------------------------------*
 * File: AesOperation.h
 *
 * AES(Advanced Encryption Standard) proguram for windows
 *
 * Created date: 2015.07.30
 *
 * Copyright (c) 2015 Shinji Wakai
 *------------------------------------------------------------------------------*/
#pragma once

#ifdef __cplusplus
    #define EXTERN_C    extern "C"
#else
    #define EXTERN_C    extern
#endif

#ifdef	_AES_OPERATION_CPP_
#define DLLAPI EXTERN_C __declspec(dllexport)
#else
#define DLLAPI EXTERN_C __declspec(dllimport)
#endif

#define AES_BLK_BYTE		(16)
#define AES_BLK_SIZE		(AES_BLK_BYTE / sizeof(ULONG))			/* 4 */

#define	MAX_ROUND			(14)
#define	MAX_EXT_KEY_SIZE	(AES_BLK_SIZE * (MAX_ROUND + 1))		/* 60 */

typedef	enum	_PROC_TYPES{
	PROC_X86_X64 = 0,
	PROC_SSE,
	PROC_SSE2,
	PROC_SSSE3,
	PROC_AES_NI,
	PROC_AUTO,
	PROC_TYPE_MIN = PROC_X86_X64,
	PROC_TYPE_MAX = PROC_AUTO
}PROC_TYPES;

typedef	enum	_AES_TYPES{
	AES_TYPE_128 = 0,
	AES_TYPE_192 = 2,
	AES_TYPE_256 = 4,
	AES_TYPE_MIN = AES_TYPE_128,
	AES_TYPE_MAX = AES_TYPE_256
}AES_TYPES;

typedef	struct	_PROC_STATUS{
	ULONG	SSE1  : 1;
	ULONG	SSE2  : 1;
	ULONG	SSE3  : 1;
	ULONG	SSSE3 : 1;
	ULONG	AESNI : 1;
}PROC_STATUS;

typedef	enum	_AES_STATUS{
	AES_STATUS_OK = 0,
	AES_STATUS_NG = -1
}AES_STATUS;

DLLAPI AES_STATUS AES_getProcStatus(PROC_STATUS *pProcStatus);
DLLAPI AES_STATUS AES_keyExpansion(PROC_TYPES proc,AES_TYPES type,const void *pKey,ULONG pExtKey[MAX_EXT_KEY_SIZE]);
DLLAPI AES_STATUS AES_CBC_encryptExp(PROC_TYPES proc,AES_TYPES type,const ULONG pExtKey[MAX_EXT_KEY_SIZE],const void *pCbcInt,const void *pSrc,void *pDst,ULONG size);
DLLAPI AES_STATUS AES_CBC_decryptExp(PROC_TYPES proc,AES_TYPES type,const ULONG pExtKey[MAX_EXT_KEY_SIZE],const void *pCbcInt,const void *pSrc,void *pDst,ULONG size);
DLLAPI AES_STATUS AES_CBC_encrypt(PROC_TYPES proc,AES_TYPES type,const void *pKey,const void *pCbcInt,const void *pSrc,void *pDst,ULONG size);
DLLAPI AES_STATUS AES_CBC_decrypt(PROC_TYPES proc,AES_TYPES type,const void *pKey,const void *pCbcInt,const void *pSrc,void *pDst,ULONG size);
DLLAPI AES_STATUS AES_CBC_mac(PROC_TYPES proc,AES_TYPES type,const void *pKey,const void *pCbcInt,const void *pSrc,void *pDst,ULONG size);
DLLAPI AES_STATUS AES_ECB_encryptExp(PROC_TYPES proc,AES_TYPES type,const ULONG pExtKey[MAX_EXT_KEY_SIZE],const void *pSrc,void *pDst,ULONG size);
DLLAPI AES_STATUS AES_ECB_decryptExp(PROC_TYPES proc,AES_TYPES type,const ULONG pExtKey[MAX_EXT_KEY_SIZE],const void *pSrc,void *pDst,ULONG size);
DLLAPI AES_STATUS AES_ECB_encrypt(PROC_TYPES proc,AES_TYPES type,const void *pKey,const void *pSrc,void *pDst,ULONG size);
DLLAPI AES_STATUS AES_ECB_decrypt(PROC_TYPES proc,AES_TYPES type,const void *pKey,const void *pSrc,void *pDst,ULONG size);
DLLAPI AES_STATUS AES_CTR_encryptExp(PROC_TYPES proc,AES_TYPES type,const ULONG pExtKey[MAX_EXT_KEY_SIZE],const UCHAR pIvec[8],const UCHAR pNonce[4],const void *pSrc,void *pDst,ULONG size,ULONG count);
DLLAPI AES_STATUS AES_CTR_encrypt(PROC_TYPES proc,AES_TYPES type,const void *pKey,const UCHAR pIvec[8],const UCHAR pNonce[4],const void *pSrc,void *pDst,ULONG size,ULONG count);

#define CAM_BLK_BYTE		AES_BLK_BYTE
#define CAM_BLK_SIZE		(CAM_BLK_BYTE / sizeof(ULONG))			/* 4 */
#define	CAM_EXT_KEY_SIZE	(CAM_BLK_SIZE * (17))					/* 68 */

typedef	enum	_CAM_TYPES{
	CAM_TYPE_128 = 0,
	CAM_TYPE_192,
	CAM_TYPE_256,
	CAM_TYPE_MIN = CAM_TYPE_128,
	CAM_TYPE_MAX = CAM_TYPE_256
}CAM_TYPES;

typedef	enum	_CAM_STATUS{
	CAM_STATUS_OK = 0,
	CAM_STATUS_NG = -1
}CAM_STATUS;

DLLAPI CAM_STATUS CAM_keyExpansion(CAM_TYPES type,const void *pKey,unsigned long pExtKey[CAM_EXT_KEY_SIZE]);
DLLAPI CAM_STATUS CAM_CBC_encryptExp(CAM_TYPES type,const unsigned long pExtKey[CAM_EXT_KEY_SIZE],const void *pCbcInt,const void *pSrc,void *pDst,unsigned long size);
DLLAPI CAM_STATUS CAM_CBC_decryptExp(CAM_TYPES type,const unsigned long pExtKey[CAM_EXT_KEY_SIZE],const void *pCbcInt,const void *pSrc,void *pDst,unsigned long size);
DLLAPI CAM_STATUS CAM_CBC_encrypt(CAM_TYPES type,const void *pKey,const void *pCbcInt,const void *pSrc,void *pDst,unsigned long size);
DLLAPI CAM_STATUS CAM_CBC_decrypt(CAM_TYPES type,const void *pKey,const void *pCbcInt,const void *pSrc,void *pDst,unsigned long size);
DLLAPI CAM_STATUS CAM_CBC_mac(CAM_TYPES type,const void *pKey,const void *pCbcInt,const void *pSrc,void *pDst,unsigned long size);
DLLAPI CAM_STATUS CAM_ECB_encryptExp(CAM_TYPES type,const unsigned long pExtKey[CAM_EXT_KEY_SIZE],const void *pSrc,void *pDst,unsigned long size);
DLLAPI CAM_STATUS CAM_ECB_decryptExp(CAM_TYPES type,const unsigned long pExtKey[CAM_EXT_KEY_SIZE],const void *pSrc,void *pDst,unsigned long size);
DLLAPI CAM_STATUS CAM_ECB_encrypt(CAM_TYPES type,const void *pKey,const void *pSrc,void *pDst,unsigned long size);
DLLAPI CAM_STATUS CAM_ECB_decrypt(CAM_TYPES type,const void *pKey,const void *pSrc,void *pDst,unsigned long size);

DLLAPI CAM_STATUS CAM_CTR_encryptExp(CAM_TYPES type,const unsigned long pExtKey[CAM_EXT_KEY_SIZE],const unsigned char ivec[8],const unsigned char nonce[4],const void *pSrc,void *pDst,unsigned long size,ULONG count);
DLLAPI CAM_STATUS CAM_CTR_encrypt(CAM_TYPES type,const void *pKey,const unsigned char ivec[8],const unsigned char nonce[4],const void *pSrc,void *pDst,unsigned long size,ULONG count);
